rm(list=ls())
library("rtracklayer")
library(ggplot2)
library(ggpubr)
library(dplyr)
library(cowplot)
library(svglite)

generate_cna_plots<-function(dat, dat_cnv, chrs, max_cn = 6, prop_samples = 0.1)
{
  dat <- subset(dat, Ratio > 0)
  names(dat_cnv)<-c("Chromosome", "Start", "End", "CopyNumber", "Type")
  
  # we will plot copy number up to max_cn
  dat<-dat[dat$CopyNumber <= max_cn,] 
  
  cols <- c("red", "blue")
  plots<-list()
  for (chr in chrs)
  {
    chr_dat<-subset(dat, Chromosome==chr)
    chr_cnv<-subset(dat_cnv, Chromosome==chr)
    N <- dim(chr_dat)[1]
    n_samples <- round(N*prop_samples)
    if (N > n_samples) {
      sample_df<-sample_n(chr_dat, n_samples)
    } else {
      sample_df<-chr_dat
    }
    p <- ggplot() + geom_point(data = sample_df, aes(x = Start, y = 2*Ratio), alpha=0.5, col='gray')
    p <- p + theme_bw()
    p <- p + xlab(chr)
    p <- p + theme(axis.text.x = element_blank(), axis.title.x = element_text(size=60, face="bold"), axis.ticks.x = element_blank())
    if (chr == 1) {
      p <- p + theme(axis.title.y = element_blank(), axis.ticks.y = element_blank(), axis.text.y = element_text(size=40, face="bold"))
    } else {
      p <- p + theme(axis.title.y = element_blank(), axis.ticks.y = element_blank(), axis.text.y = element_blank())
    }
    p <- p + theme(panel.border = element_blank(), panel.grid.minor = element_blank())
    p <- p + theme(panel.grid.major.y = element_line( linetype = 2, color="gray" ))
    p <- p + theme(panel.grid.major.x = element_blank())
    p <- p + ylim(0, max_cn) 
    p <- p + geom_segment(data=chr_cnv, aes(x=Start, xend=End, y=CopyNumber, yend=CopyNumber), col=cols[chr_cnv$Type], size=1)
    p <- p + theme(legend.position="none")
    plots[[chr]] <- p
  }
  plots
}

data_root <- "../"
chrs <- c(1:22, "X", "Y")

data_dirs<-matrix(c(127, 128, 130, 135, 136, 138, 143, 144, 146, 151, 152, 154, 167, 168, 170), ncol = 3, byrow = T)
patients<-1:5

for (patient_id in patients)
{
  patient_name <- paste("P", patient_id, sep="")
  print(patient_name)
  data_path <- paste(data_root, patient_name, sep="/")

  common_name <- paste("PXXXX_", data_dirs[patient_id, 1], sep="")
  time0<-read.table(paste(data_path, paste(common_name, "/05-CNA/", common_name, ".clean.dedup.bam_ratio.txt", sep=""), sep="/"), header=T)
  time0_cnv<-read.table(paste(data_path, paste(common_name, "/05-CNA/", common_name, ".clean.dedup.bam_CNVs", sep=""), sep="/"), header=F)

  common_name <- paste("PXXXX_", data_dirs[patient_id, 2], sep="")
  ot<-read.table(paste(data_path, paste(common_name, "/05-CNA/", common_name, ".clean.dedup.bam_ratio.txt", sep=""), sep="/"), header=T)
  ot_cnv<-read.table(paste(data_path, paste(common_name, "/05-CNA/", common_name, ".clean.dedup.bam_CNVs", sep=""), sep="/"), header=F)

  common_name <- paste("PXXXX_", data_dirs[patient_id, 3], sep="")
  tumor<-read.table(paste(data_path, paste(common_name, "/05-CNA/", common_name, ".clean.dedup.bam_ratio.txt", sep=""), sep="/"), header=T)
  tumor_cnv<-read.table(paste(data_path, paste(common_name, "/05-CNA/", common_name, ".clean.dedup.bam_CNVs", sep=""), sep="/"), header=F)

  time0_plots <- generate_cna_plots(time0, time0_cnv, chrs)
  pp_time0 <- plot_grid(plotlist = time0_plots, align = "h", nrow = 1, ncol = length(chrs), axis = "b")
  title <- ggdraw() + draw_label(paste("DNA", patient_name, "Scraping", sep="_"), fontface='bold', size=60)
  pp <- plot_grid(title, pp_time0, nrow = 2, rel_heights = c(0.4, 1))
  save_plot(paste(patient_name, "CNA_time0.pdf", sep="_"), pp, ncol = length(chrs), nrow = 2, limitsize = FALSE)
  save_plot(paste(patient_name, "CNA_time0.svg", sep="_"), pp, ncol = length(chrs), nrow = 2, limitsize = FALSE)

  tumor_plots <- generate_cna_plots(tumor, tumor_cnv, chrs)
  pp_tumor <- plot_grid(plotlist = tumor_plots, align = "h", nrow = 1, ncol = length(chrs), axis = "b")
  title <- ggdraw() + draw_label(paste("DNA", patient_name, "Tumor", sep="_"), fontface='bold', size=60)
  pp <- plot_grid(title, pp_tumor, nrow = 2, rel_heights = c(0.4, 1))
  save_plot(paste(patient_name, "CNA_tumor.pdf", sep="_"), pp, ncol = length(chrs), nrow = 2, limitsize = FALSE)
  save_plot(paste(patient_name, "CNA_tumor.svg", sep="_"), pp, ncol = length(chrs), nrow = 2, limitsize = FALSE)

  ot_plots <- generate_cna_plots(ot, ot_cnv, chrs)
  pp_o <- plot_grid(plotlist = ot_plots, align = "h", nrow = 1, ncol = length(chrs), axis = "b")
  title <- ggdraw() + draw_label(paste("DNA", patient_name, "WTC", sep="_"), fontface='bold', size=60)
  pp <- plot_grid(title, pp_o, nrow = 2, rel_heights = c(0.4, 1))
  save_plot(paste(patient_name, "CNA_organoid.pdf", sep="_"), pp, ncol = length(chrs), nrow = 2, limitsize = FALSE)
  save_plot(paste(patient_name, "CNA_organoid.svg", sep="_"), pp, ncol = length(chrs), nrow = 2, limitsize = FALSE)
}



